@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
+// SPDX-FileCopyrightText: 2026 UnionTech Software Technology Co., Ltd.
 //
 // SPDX-License-Identifier: LGPL-3.0-or-later
 #include "applicationadaptor.h"
@@ -12,9 +12,12 @@
 #include "applicationHooks.h"
 #include "desktopfilegenerator.h"
 #include <QFile>
+#include <QGuiApplication>
+#include <QLoggingCategory>
 #include <QHash>
 #include <QDBusMessage>
 #include <QStringBuilder>
+#include <DUtil>
 #include <unistd.h>
 
 ApplicationManager1Service::~ApplicationManager1Service() = default;
@@ -24,8 +27,27 @@ ApplicationManager1Service::ApplicationManager1Service(std::unique_ptr<Identifie
     : m_identifier(std::move(ptr))
     , m_storage(std::move(storage))
 {
+    // Initialize prelaunch splash helper only when running on Wayland.
+    bool isWayland = false;
+    if (auto *app = qobject_cast<QGuiApplication *>(QCoreApplication::instance())) {
+        if (app->nativeInterface<QNativeInterface::QWaylandApplication>() != nullptr) {
+            isWayland = true;
+        }
+    }
+
+    if (isWayland) {
+        m_splashHelper.reset(new (std::nothrow) PrelaunchSplashHelper());
+        if (!m_splashHelper) {
+            qCWarning(amPrelaunchSplash) << "Failed to allocate PrelaunchSplashHelper.";
+        } else {
+            qCInfo(amPrelaunchSplash) << "PrelaunchSplashHelper initialized.";
+        }
+    } else {
+        qCInfo(amPrelaunchSplash) << "Skip PrelaunchSplashHelper (not running on Wayland)";
+    }
+
     using namespace std::chrono_literals;
-    m_reloadTimer.setInterval(50);
+    m_reloadTimer.setInterval(500);
     m_reloadTimer.setSingleShot(true);
     connect(&m_reloadTimer, &QTimer::timeout, this, &ApplicationManager1Service::doReloadApplications);
 }
@@ -65,8 +87,20 @@ void ApplicationManager1Service::initService(QDBusConnection &connection) noexce
     }
 
     connect(&m_watcher, &QFileSystemWatcher::directoryChanged, this, &ApplicationManager1Service::ReloadApplications);
-    auto unhandled = m_watcher.addPaths(getDesktopFileDirs());
-    for (const auto &dir : unhandled) {
+
+    // Ensure all directories exist before adding watches
+    auto desktopDirs = getDesktopFileDirs();
+    for (const auto &dirPath : std::as_const(desktopDirs)) {
+        QDir dir(dirPath);
+        if (!dir.exists()) {
+            if (!dir.mkpath(dirPath)) {
+                qWarning() << "Failed to create directory:" << dirPath;
+            }
+        }
+    }
+
+    auto unhandled = m_watcher.addPaths(desktopDirs);
+    for (const auto &dir : std::as_const(unhandled)) {
         qCritical() << "couldn't watch directory:" << dir;
     }
 
@@ -87,6 +121,7 @@ void ApplicationManager1Service::initService(QDBusConnection &connection) noexce
             if (!app) {
                 return;
             }
+            void updateAutostartStatus() noexcept;
 
             // 服务在 AM 之后启动那么 instance size 是 0， newJob 时尝试添加一次
             // 比如 dde-file-manager.service 如果启动的比 AM 晚，那么在 scanInstances 时不会 addInstanceToApplication
@@ -120,7 +155,7 @@ void ApplicationManager1Service::initService(QDBusConnection &connection) noexce
     if (ret.type() == QDBusMessage::ErrorMessage) {
         qFatal("%s", ret.errorMessage().toLocal8Bit().data());
     }
-    envToPath(qdbus_cast<QStringList>(ret.arguments().first().value<QDBusVariant>().variant()));
+    envToPath(qdbus_cast<QStringList>(ret.arguments().constFirst().value<QDBusVariant>().variant()));
 
     auto sysBus = QDBusConnection::systemBus();
     if (!sysBus.connect("org.desktopspec.ApplicationUpdateNotifier1",
@@ -136,10 +171,10 @@ void ApplicationManager1Service::initService(QDBusConnection &connection) noexce
 
     scanApplications();
 
-    auto needLaunch = scanAutoStart();
-
     scanInstances();
 
+    updateAutostartStatus();
+
     auto storagePtr = m_storage.lock();
     if (!storagePtr->setFirstLaunch(false)) {
         qCritical() << "failed to update state of application manager, some properties may be lost after restart.";
@@ -173,51 +208,6 @@ void ApplicationManager1Service::initService(QDBusConnection &connection) noexce
     if (flag.open(QFile::WriteOnly | QFile::Truncate)) {
         flag.write(sessionId, sessionId.size());
     }
-    auto value = QString::fromLocal8Bit(qgetenv("XDG_SESSION_TYPE"));
-    if (!value.isEmpty() && value == QString::fromLocal8Bit("wayland")) {
-        for (const auto &[app, realExec] : needLaunch.asKeyValueRange()) {
-            app->Launch({}, {}, {}, realExec);
-        }
-    } else {
-        constexpr auto XSettings = "org.deepin.dde.XSettings1";
-
-        auto *watcher = new (std::nothrow)
-            QDBusServiceWatcher{XSettings, QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForRegistration, this};
-
-        auto *sigCon = new (std::nothrow) QMetaObject::Connection{};
-
-        auto singleSlot = [watcher, sigCon, autostartMap = std::move(needLaunch)]() {
-            QObject::disconnect(*sigCon);
-            delete sigCon;
-            qDebug() << XSettings << "is registered.";
-
-            for (const auto &[app, realExec] : autostartMap.asKeyValueRange()) {
-                app->Launch({}, {}, {}, realExec);
-            }
-
-            watcher->deleteLater();
-        };
-
-        if (watcher != nullptr) {
-            *sigCon = connect(watcher, &QDBusServiceWatcher::serviceRegistered, singleSlot);
-        }
-
-        auto msg = QDBusMessage::createMethodCall(
-            "org.freedesktop.DBus", "/org/freedesktop/DBus", "org.freedesktop.DBus", "NameHasOwner");
-        msg << XSettings;
-
-        auto reply = QDBusConnection::sessionBus().call(msg);
-        if (reply.type() != QDBusMessage::ReplyMessage) {
-            qWarning() << "call org.freedesktop.DBus::NameHasOwner failed, skip autostart:" << reply.errorMessage();
-            // The connection should not be deleted, failure to call org.freedesktop.DBus::NameHasOwner does not mean that the
-            // XSettings service is invalid.
-            return;
-        }
-
-        if (reply.arguments().first().toBool()) {
-            singleSlot();
-        }
-    }
 }
 
 void ApplicationManager1Service::addInstanceToApplication(const QString &unitName, const QDBusObjectPath &systemdUnitPath)
@@ -254,8 +244,6 @@ void ApplicationManager1Service::removeInstanceFromApplication(const QString &un
 {
     auto info = processUnitName(unitName);
     auto appId = std::move(info.applicationID);
-    auto launcher = std::move(info.Launcher);
-    auto instanceId = std::move(info.instanceID);
 
     if (appId.isEmpty()) {
         return;
@@ -287,7 +275,7 @@ void ApplicationManager1Service::removeInstanceFromApplication(const QString &un
 
 void ApplicationManager1Service::scanMimeInfos() noexcept
 {
-    const auto& dirs = getMimeDirs();
+    const auto &dirs = getMimeDirs();
     for (const auto &dir : dirs) {
         auto info = MimeInfo::createMimeInfo(dir);
         if (info) {
@@ -296,6 +284,13 @@ void ApplicationManager1Service::scanMimeInfos() noexcept
     }
 }
 
+void ApplicationManager1Service::reloadMimeInfos() noexcept
+{
+    m_mimeManager->reset();
+    scanMimeInfos();
+    emit m_mimeManager->MimeInfoReloaded();
+}
+
 void ApplicationManager1Service::scanApplications() noexcept
 {
     const auto &desktopFileDirs = getDesktopFileDirs();
@@ -340,17 +335,16 @@ void ApplicationManager1Service::scanInstances() noexcept
         return;
     }
 
-    auto v = result.arguments().first();
+    auto v = result.arguments().constFirst();
     QList<SystemdUnitDBusMessage> units;
     v.value<QDBusArgument>() >> units;
-    for (const auto &unit : units) {
+    for (const auto &unit : std::as_const(units)) {
         this->addInstanceToApplication(unit.name, unit.objectPath);
     }
 }
 
-QHash<QSharedPointer<ApplicationService>, QString> ApplicationManager1Service::scanAutoStart() noexcept
+void ApplicationManager1Service::updateAutostartStatus() noexcept
 {
-    QHash<QSharedPointer<ApplicationService>, QString> ret;
     auto autostartDirs = getAutoStartDirs();
     std::map<QString, DesktopFile> autostartItems;
 
@@ -389,8 +383,6 @@ QHash<QSharedPointer<ApplicationService>, QString> ApplicationManager1Service::s
 
         QSharedPointer<ApplicationService> app{nullptr};
         auto asApplication = tmp.value(DesktopFileEntryKey, X_Deepin_GenerateSource).value_or(DesktopEntry::Value{});
-        auto shouldLaunch = tmp.value(DesktopFileEntryKey, DesktopEntryHidden).value_or(DesktopEntry::Value{});
-        auto autostartFlag = shouldLaunch.isNull() || (shouldLaunch.toString().compare("true", Qt::CaseInsensitive) != 0);
 
         if (!asApplication.isNull()) {  // modified by application manager
             auto appSource = asApplication.toString();
@@ -423,13 +415,6 @@ QHash<QSharedPointer<ApplicationService>, QString> ApplicationManager1Service::s
             }
 
             app->setAutostartSource({desktopFile.sourcePath(), std::move(tmp)});
-            if (autostartFlag) {
-                auto val = tmp.value(DesktopFileEntryKey, "Exec").value_or(DesktopEntry::Value{});
-                auto realExec = val.isNull() ? QString{} : val.toString();
-                qInfo() << "launch autostart application " << app->id() << " by " << realExec;
-                ret.insert(app, realExec);
-            }
-
             continue;
         }
 
@@ -440,13 +425,6 @@ QHash<QSharedPointer<ApplicationService>, QString> ApplicationManager1Service::s
         }
 
         if (auto existApp = m_applicationList.value(desktopFile.desktopId()); existApp) {
-            if (autostartFlag) {
-                auto val = tmp.value(DesktopFileEntryKey, "Exec").value_or(DesktopEntry::Value{});
-                auto realExec = val.isNull() ? QString{} : val.toString();
-                qInfo() << "launch exist autostart application " << existApp->id() << " by " << realExec;
-                ret.insert(existApp, realExec);
-            }
-
             existApp->setAutostartSource({desktopFile.sourcePath(), std::move(tmp)});
             continue;
         }
@@ -460,14 +438,7 @@ QHash<QSharedPointer<ApplicationService>, QString> ApplicationManager1Service::s
             continue;
         }
         app->setAutostartSource({originalSource, {}});
-
-        if (autostartFlag) {
-            qInfo() << "launch new autostart application " << app->id();
-            ret.insert(app, {});
-        }
     }
-
-    return ret;
 }
 
 void ApplicationManager1Service::loadHooks() noexcept
@@ -499,8 +470,8 @@ void ApplicationManager1Service::loadHooks() noexcept
 QList<QDBusObjectPath> ApplicationManager1Service::list() const
 {
     QList<QDBusObjectPath> paths;
-    for (const auto &appId : m_applicationList.keys())
-        paths << QDBusObjectPath{getObjectPathFromAppId(appId)};
+    for (auto it = m_applicationList.constBegin(); it != m_applicationList.constEnd(); ++it)
+        paths << QDBusObjectPath{getObjectPathFromAppId(it.key())};
 
     return paths;
 }
@@ -541,7 +512,7 @@ QSharedPointer<ApplicationService> ApplicationManager1Service::addApplication(De
 void ApplicationManager1Service::removeOneApplication(const QString &appId) noexcept
 {
     auto objectPath = QDBusObjectPath{getObjectPathFromAppId(appId)};
-    if (auto it = m_applicationList.find(appId); it != m_applicationList.cend()) {
+    if (auto it = m_applicationList.constFind(appId); it != m_applicationList.cend()) {
         emit InterfacesRemoved(objectPath, getChildInterfacesFromObject(it->data()));
         if (auto ptr = m_storage.lock(); ptr) {
             if (!ptr->deleteApplication(appId)) {
@@ -558,8 +529,8 @@ void ApplicationManager1Service::removeOneApplication(const QString &appId) noex
 
 void ApplicationManager1Service::removeAllApplication() noexcept
 {
-    for (const auto &appId : m_applicationList.keys()) {
-        removeOneApplication(appId);
+    for (auto it = m_applicationList.constBegin(); it != m_applicationList.constEnd(); ++it) {
+        removeOneApplication(it.key());
     }
 }
 
@@ -681,13 +652,12 @@ void ApplicationManager1Service::doReloadApplications()
         {"*.desktop"},
         QDir::Name | QDir::DirsLast);
 
-    for (const auto &appId : appIds) {
+    for (const auto &appId : std::as_const(appIds)) {
         removeOneApplication(appId);
     }
 
-    m_mimeManager->reset();
-    scanMimeInfos();
-    scanAutoStart();
+    reloadMimeInfos();
+    updateAutostartStatus();
 }
 
 ObjectMap ApplicationManager1Service::GetManagedObjects() const
@@ -724,12 +694,25 @@ QString ApplicationManager1Service::addUserApplication(const QVariantMap &deskto
     }
 
     QDir xdgDataHome;
-    QString dir{getXDGDataHome() + "/applications"};
+    QString dir{getXDGDataHome()};
+    // Ensure consistent path format (with trailing separator) to match getDesktopFileDirs()
+    if (!dir.endsWith(QDir::separator())) {
+        dir.append(QDir::separator());
+    }
+    dir.append("applications");
+    const bool dirExisted = QDir(dir).exists();
     if (!xdgDataHome.mkpath(dir)) {
         safe_sendErrorReply(QDBusError::Failed, "couldn't create directory of user applications.");
         return {};
     }
 
+    // If directory is newly created, add watch for it dynamically
+    if (!dirExisted && !m_watcher.directories().contains(dir)) {
+        if (!m_watcher.addPath(dir)) {
+            qWarning() << "Failed to add watch for newly created directory:" << dir;
+        }
+    }
+
     // 判断当前是否已经存在了desktop文件
     xdgDataHome.setPath(dir);
     const auto &filePath = xdgDataHome.filePath(name);
@@ -839,3 +822,115 @@ void ApplicationManager1Service::deleteUserApplication(const QString &app_id) no
 
     m_mimeManager->updateMimeCache(dir);
 }
+
+QDBusObjectPath ApplicationManager1Service::executeCommand(const QString &program,
+                                                  const QStringList &arguments,
+                                                  const QString &type,
+                                                  const QString &runId,
+                                                  const QMap<QString, QString> &envVars,
+                                                  const QString &workdir) noexcept
+{
+    // Validate required parameters
+    if (program.isEmpty()) {
+        safe_sendErrorReply(QDBusError::Failed, "program path cannot be empty.");
+        return QDBusObjectPath("/");
+    }
+
+    // Validate execution type
+    const QStringList validTypes = {"shortcut", "script", "portablebinary"};
+    if (!validTypes.contains(type)) {
+        safe_sendErrorReply(QDBusError::Failed, QString{"Invalid type '%1'. Must be one of: shortcut, script, portablebinary"}.arg(type));
+        return QDBusObjectPath("/");
+    }
+
+    // Check if program exists and is executable
+    QFileInfo programInfo(program);
+    if (!programInfo.exists()) {
+        safe_sendErrorReply(QDBusError::Failed, QString{"Program '%1' does not exist."}.arg(program));
+        return QDBusObjectPath("/");
+    }
+    if (!programInfo.isExecutable()) {
+        safe_sendErrorReply(QDBusError::Failed, QString{"Program '%1' is not executable."}.arg(program));
+        return QDBusObjectPath("/");
+    }
+
+    // Generate runId if not provided (empty string)
+    QString actualRunId = runId;
+    if (actualRunId.isEmpty()) {
+        // Escape the program path to create a valid runId
+        actualRunId = program;
+    }
+    actualRunId = DUtil::escapeToObjectPath(actualRunId);
+
+    // Generate random component for systemd unit name
+    QString randomComponent = QUuid::createUuid().toString(QUuid::Id128).mid(1, 8);
+
+    // Construct systemd unit name according to specification
+    QString unitName = QString{"app-DDE-tmp.%1.%2@%3.service"}.arg(type, actualRunId, randomComponent);
+
+    // Prepare the command line
+    QStringList commandLine;
+    commandLine << program;
+    commandLine << arguments;
+
+    // Add system environment variables
+    QStringList environment;
+    QProcessEnvironment systemEnv = QProcessEnvironment::systemEnvironment();
+    const auto &keys = systemEnv.keys();
+    for (const QString &key : std::as_const(keys)) {
+        environment << QString{"%1=%2"}.arg(key, systemEnv.value(key));
+    }
+    for (auto it = envVars.constBegin(); it != envVars.constEnd(); ++it) {
+        environment << QString{"%1=%2"}.arg(it.key(), it.value());
+    }
+
+    QList<SystemdProperty> properties;
+    // 1. Property: Description
+    properties.append({ "Description", QDBusVariant(QString("Run: %1").arg(program)) });
+
+    // 2. Property: ExecStart (Type: a(sasb))
+    // Systemd 要求 ExecStart 是一个结构体数组，因为一个服务可以有多个 ExecStart 命令
+    SystemdExecCommand execCmd;
+    execCmd.path = program;     // 二进制路径
+    execCmd.args = commandLine; // 完整参数列表 (argv)
+    execCmd.unclean = false;    // 是否忽略非零返回值
+
+    QList<SystemdExecCommand> execStartList;
+    execStartList << execCmd;
+    properties.append({ "ExecStart", QDBusVariant(QVariant::fromValue(execStartList)) });
+
+    // 3. Property: Environment (Type: as)
+    properties.append({ "Environment", QDBusVariant(environment) });
+
+    // 4. Property: WorkingDirectory (Type: s)
+    if (!workdir.isEmpty()) {
+        properties.append({ "WorkingDirectory", QDBusVariant(workdir) });
+    }
+
+    // Call systemd to start the service
+    // StartTransientUnit signature: (s, s, a(sv), a(sa(sv)))
+    // args: unit_name, mode("replace"), properties, aux_units
+
+    QDBusConnection conn = QDBusConnection::sessionBus();
+    auto msg = QDBusMessage::createMethodCall("org.freedesktop.systemd1", // Service
+                                              "/org/freedesktop/systemd1", // Path
+                                              "org.freedesktop.systemd1.Manager", // Interface
+                                              "StartTransientUnit");
+
+    msg.setArguments({
+        unitName,                           // arg1: name
+        "replace",                          // arg2: mode
+        QVariant::fromValue(properties),    // arg3: properties a(sv)
+        QVariant::fromValue(QList<SystemdAux>()) // arg4: aux units (empty)
+    });
+
+    auto reply = conn.asyncCall(msg);
+    qInfo() << "Request sent to start transient unit (Async):" << unitName;
+
+    // TODO: the return value is currently empty, it's reserved for future use if we plan to make the spawned process as an AM-managed instance
+    // At that moment, we should:
+    // 1. Add a new API to JobManager1Service to allow add/start a new job without providing an application D-Bus path
+    // 2. Use JobManager1Service::addJob to add a new job and run it, like what we do in ApplicationService::Launch()
+    // 3. Return the object path that the new JobManager service offered.
+    return QDBusObjectPath("/");
+}
