@@ -1,32 +1,32 @@
-// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
+// SPDX-FileCopyrightText: 2023 - 2026 UnionTech Software Technology Co., Ltd.
 //
 // SPDX-License-Identifier: LGPL-3.0-or-later
 
 #ifndef GLOBAL_H
 #define GLOBAL_H
 
-#include <QString>
-#include <QMap>
+#include "config.h"
+#include "constant.h"
+#include <QDBusArgument>
 #include <QDBusConnection>
-#include <optional>
 #include <QDBusError>
-#include <QMetaType>
-#include <QMetaClassInfo>
-#include <QLocale>
-#include <QDir>
-#include <QRegularExpression>
+#include <QDBusMessage>
 #include <QDBusObjectPath>
 #include <QDBusUnixFileDescriptor>
-#include <QDBusArgument>
-#include <QDBusMessage>
-#include <unistd.h>
-#include <QUuid>
+#include <QDir>
+#include <QLocale>
 #include <QLoggingCategory>
+#include <QMap>
+#include <QMetaClassInfo>
+#include <QMetaType>
+#include <QRegularExpression>
+#include <QString>
+#include <QUuid>
+#include <csignal> // IWYU pragma: keep
+#include <optional>
 #include <sys/stat.h>
 #include <sys/syscall.h>
-#include <signal.h>
-#include "constant.h"
-#include "config.h"
+#include <unistd.h>
 
 Q_DECLARE_LOGGING_CATEGORY(DDEAMProf)
 
@@ -47,6 +47,75 @@ struct SystemdUnitDBusMessage
     QDBusObjectPath objectPath;
 };
 
+// --- Systemd D-Bus 类型定义开始 ---
+
+// 对应 Systemd ExecStart 的结构: (path, argv, ignore_failure) -> (sasb)
+struct SystemdExecCommand {
+    QString path;
+    QStringList args;
+    bool unclean; // 是否忽略错误
+};
+Q_DECLARE_METATYPE(SystemdExecCommand)
+
+// 对应 Systemd 属性结构: (name, value) -> (sv)
+struct SystemdProperty {
+    QString name;
+    QDBusVariant value; // 使用 QDBusVariant 来明确这是一个 Variant 容器
+};
+Q_DECLARE_METATYPE(SystemdProperty)
+
+// 对应 Systemd 辅助单元结构 (Aux Unit): (name, properties) -> (sa(sv))
+// StartTransientUnit 的第4个参数需要这个类型的数组，即使它是空的
+struct SystemdAux {
+    QString name;
+    QList<SystemdProperty> properties;
+};
+Q_DECLARE_METATYPE(SystemdAux)
+
+// 序列化操作符重载
+inline QDBusArgument &operator<<(QDBusArgument &arg, const SystemdExecCommand &cmd) {
+    arg.beginStructure();
+    arg << cmd.path << cmd.args << cmd.unclean;
+    arg.endStructure();
+    return arg;
+}
+
+inline const QDBusArgument &operator>>(const QDBusArgument &arg, SystemdExecCommand &cmd) {
+    arg.beginStructure();
+    arg >> cmd.path >> cmd.args >> cmd.unclean;
+    arg.endStructure();
+    return arg;
+}
+
+inline QDBusArgument &operator<<(QDBusArgument &arg, const SystemdProperty &prop) {
+    arg.beginStructure();
+    arg << prop.name << prop.value;
+    arg.endStructure();
+    return arg;
+}
+
+inline const QDBusArgument &operator>>(const QDBusArgument &arg, SystemdProperty &prop) {
+    arg.beginStructure();
+    arg >> prop.name >> prop.value;
+    arg.endStructure();
+    return arg;
+}
+
+inline QDBusArgument &operator<<(QDBusArgument &arg, const SystemdAux &aux) {
+    arg.beginStructure();
+    arg << aux.name << aux.properties;
+    arg.endStructure();
+    return arg;
+}
+
+inline const QDBusArgument &operator>>(const QDBusArgument &arg, SystemdAux &aux) {
+    arg.beginStructure();
+    arg >> aux.name >> aux.properties;
+    arg.endStructure();
+    return arg;
+}
+// --- Systemd D-Bus 类型定义结束 ---
+
 inline const QDBusArgument &operator>>(const QDBusArgument &argument, QStringMap &map)
 {
     argument.beginMap();
@@ -80,7 +149,7 @@ inline const QDBusArgument &operator>>(const QDBusArgument &argument, QList<Syst
     return argument;
 }
 
-inline QString getApplicationLauncherBinary()
+inline const QString& getApplicationLauncherBinary()
 {
     static const QString bin = []() {
         auto value = qgetenv("DEEPIN_APPLICATION_MANAGER_APP_LAUNCH_HELPER_BIN");
@@ -315,14 +384,20 @@ inline QString escapeToObjectPath(const QString &str)
     if (str.isEmpty()) {
         return "_";
     }
-
     auto ret = str;
-    QRegularExpression re{R"([^a-zA-Z0-9])"};
+
+    const static auto _re = [] {
+        QRegularExpression tmp{QStringLiteral(R"([^a-zA-Z0-9])")};
+        tmp.optimize();
+        return tmp;
+    }();
+    const auto re = _re;
+
     auto matcher = re.globalMatch(ret);
     while (matcher.hasNext()) {
         auto replaceList = matcher.next().capturedTexts();
         replaceList.removeDuplicates();
-        for (const auto &c : replaceList) {
+        for (const auto &c : std::as_const(replaceList)) {
             auto hexStr = QString::number(static_cast<uint>(c.front().toLatin1()), 16);
             ret.replace(c, QString{R"(_%1)"}.arg(hexStr));
         }
@@ -350,14 +425,21 @@ inline QString escapeApplicationId(const QString &id)
     }
 
     auto ret = id;
-    QRegularExpression re{R"([^a-zA-Z0-9])"};
+
+    static const auto _re = [] {
+        QRegularExpression tmp{QStringLiteral(R"([^a-zA-Z0-9])")};
+        tmp.optimize();
+        return tmp;
+    }();
+    const auto re = _re;
+
     auto matcher = re.globalMatch(ret);
     while (matcher.hasNext()) {
         auto replaceList = matcher.next().capturedTexts();
         replaceList.removeDuplicates();
-        for (const auto &c : replaceList) {
+        for (const auto &c : std::as_const(replaceList)) {
             auto hexStr = QString::number(static_cast<uint>(c.front().toLatin1()), 16);
-            ret.replace(c, QString{R"(\x%1)"}.arg(hexStr));
+            ret.replace(c, QStringLiteral(R"(\x%1)").arg(hexStr));
         }
     }
     return ret;
@@ -369,7 +451,7 @@ inline QString unescapeApplicationId(const QString &id)
     for (qsizetype i = 0; i < id.size(); ++i) {
         if (id[i] == '\\' and i + 3 < id.size()) {
             auto hexStr = id.sliced(i + 2, 2);
-            ret.replace(QString{R"(\x%1)"}.arg(hexStr), QChar::fromLatin1(hexStr.toUInt(nullptr, 16)));
+            ret.replace(QStringLiteral(R"(\x%1)").arg(hexStr), QChar::fromLatin1(hexStr.toUInt(nullptr, 16)));
             i += 3;
         }
     }
@@ -603,7 +685,7 @@ inline QByteArray getCurrentSessionId()
         return {};
     }
 
-    auto id = ret.arguments().first();
+    const auto &id = ret.arguments().constFirst();
     return id.value<QDBusVariant>().variant().toByteArray();
 }
 
